/**
 ** Parchis4A.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.graphics.PorterDuff;
import android.graphics.drawable.AnimationDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.RadioGroup;
import android.widget.Toast;

public class Parchis4A extends Activity {

    private static String ICICLE_KEY = "parchis4a-state";
//	private static final int colorj[] = {0xffffff00, 0xff0000ff, 0xffff0000, 0xff00ff00};
	private static final int colorj[] = {0xffffff55, 0xff5555ff, 0xffff5555, 0xff00aa00};

	private int autointv = 10;
	private int tipo_jugador[];
	private boolean goFast = false;
	
	private EstadoPartida mEP;
	private ParchisView mParchisView;
	private Button mTirador;
	private Button mAuxiliar;
	private Tablero mTablero;
	private RadioGroup mRG_tipo_jugador[];
	private int blinkPawn = 0;
	private Carrera mCarrera;
	private boolean enCarrera = false;
	
    static final int DIALOG_SELECTPLAYERS_ID = 0;
    static final int DIALOG_ABOUT_ID = 1;
    static final int DIALOG_HELP_ID = 2;
    static final int DIALOG_EXIT_ID = 3;
    static final int DIALOG_WELCOME_ID = 4;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        setContentView(R.layout.main);
        tipo_jugador = new int[4];
        mRG_tipo_jugador = new RadioGroup[4];
        tipo_jugador[0] = EstadoPartida.PERSONA;
        tipo_jugador[1] = EstadoPartida.MAQUINA;
        tipo_jugador[2] = EstadoPartida.MAQUINA;
        tipo_jugador[3] = EstadoPartida.MAQUINA;
        mParchisView = (ParchisView) findViewById(R.id.parchis);
//        mParchisView.setBackgroundColor(0x0000ff);
        mTirador = (Button) findViewById(R.id.btirador);
        mAuxiliar = (Button) findViewById(R.id.bauxiliar);
        if (!restoreState(savedInstanceState)) {
        	setNewGame();
            showDialog(DIALOG_WELCOME_ID);
        }
    }
    
    @Override
    public void onSaveInstanceState(Bundle outState) {
        //Store the game state
    	Bundle estado = mEP.saveState();
        estado.putBoolean("n0gf", goFast);
        estado.putInt("n0aitv", autointv);
        estado.putIntArray("n0tj", tipo_jugador);
        outState.putBundle(ICICLE_KEY, estado);
    }
    
    public boolean restoreState(Bundle savedInstanceState) {
    	if (savedInstanceState == null) return false;
        Bundle map = savedInstanceState.getBundle(ICICLE_KEY);
        if (map == null) return false;

        blinkPawn = 0;
    	enCarrera = false;
    	mAutoHandler.removeMessages(0);

    	goFast = map.getBoolean("n0gf");
        autointv = map.getInt("n0aitv");
        tipo_jugador = map.getIntArray("n0tj");
        mEP = new EstadoPartida();
        mTablero = new Tablero();
        mEP.restoreState(map);    	
        mTablero.setEstadoPartida(mEP);
        if (mEP.accion == EstadoPartida.ACC_PARADADO) {
        	mEP.accion = EstadoPartida.ACC_TIRADADO;
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE) {
            mEP.gjg = new GrupoJugadas();
            mEP.gjg.genera(mTablero, mEP.turno, mEP.vdado);
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE ||
        	mEP.accion == EstadoPartida.ACC_PASA) {
    		mTirador.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mAuxiliar.invalidate();
        	pintaVdado(mEP.vdado);
        }
        mAuxiliar.setVisibility(View.GONE);
        mParchisView.setEstadoPartida(mEP);
        mParchisView.invalidate();
        preparaSigAccion();
        return true;
    }

    public void setNewGame() {
    	blinkPawn = 0;
    	enCarrera = false;
    	mAutoHandler.removeMessages(0);

    	mEP = new EstadoPartida();
        mTablero = new Tablero();
        mEP.iniciaPartida(tipo_jugador[0], tipo_jugador[1],
        		          tipo_jugador[2], tipo_jugador[3]);
        mTablero.setEstadoPartida(mEP);
        mAuxiliar.setVisibility(View.GONE);
        mParchisView.setEstadoPartida(mEP);
        mParchisView.invalidate();
        preparaSigAccion();
   }

    private AutoHandler mAutoHandler = new AutoHandler();

    class AutoHandler extends Handler {

        @Override
        public void handleMessage(Message msg) {
            Parchis4A.this.autoUpdate();
        }

        public void sleep(long delayMillis) {
        	this.removeMessages(0);
            sendMessageDelayed(obtainMessage(0), delayMillis);
        }
    };

    public void preparaSigAccion() {
        Resources res = getBaseContext().getResources();

        if (mEP.accion == EstadoPartida.ACC_TIRADADO) {
    		mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.dadox, 0, 0, 0);
    		mTirador.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mAuxiliar.invalidate();
        	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
    		    mTirador.setText(res.getString(R.string.acc_tira));
        	} else {
    		    mTirador.setText(res.getString(R.string.acc_tiro));
        		mAutoHandler.sleep(100*autointv);
        	}
    	} else if (mEP.accion == EstadoPartida.ACC_PARADADO) {
    		mTirador.setText(res.getString(R.string.acc_espera));
    		mAutoHandler.sleep(100*autointv);
    	} else if (mEP.accion == EstadoPartida.ACC_MUEVE) {
        	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
            	mParchisView.showPawnCruz = true;
                if (mEP.gjg.jg[0].acasa)
                	mTirador.setText(res.getString(R.string.acc_acasa));
                else if (mEP.gjg.jg[0].entra) {
                	if (mEP.ha_salido[mEP.gjg.jg[0].cjugador])
                	    mTirador.setText(res.getString(R.string.acc_sales));
                	else
            	        mTirador.setText(res.getString(R.string.acc_sales2));
                } else if (mEP.gjg.njugadas > 1) {
                    mAuxiliar.setVisibility(View.VISIBLE);
        		    mTirador.setText(res.getString(R.string.acc_mueve2));
                } else
        		    mTirador.setText(res.getString(R.string.acc_mueve));
                if (mEP.gjg.njugadas > 0) {
        		    enCarrera = true;
        		    mCarrera = null;
                }
        	} else {
                if (mEP.gjg.jg[0].acasa)
                	mTirador.setText(res.getString(R.string.acc_acasam));
                else if (mEP.gjg.jg[0].entra) {
                	if (mEP.ha_salido[mEP.gjg.jg[0].cjugador])
                	    mTirador.setText(res.getString(R.string.acc_salgo));
                	else
                	    mTirador.setText(res.getString(R.string.acc_salgo2));
                } else
        		    mTirador.setText(res.getString(R.string.acc_muevo));
                if (mEP.gjg.njugadas > 0) {
                	blinkPawn = 1;
                	mParchisView.showPawnCruz = true;
        		    mAutoHandler.sleep(18*autointv);
        		    enCarrera = true;
        		    mCarrera = null;
                } else
        		    mAutoHandler.sleep(250*autointv);
        	}
            mParchisView.invalidate();
    	} else if (mEP.accion == EstadoPartida.ACC_PASA) {
        	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
        		mTirador.setText(res.getString(R.string.acc_pasas));
        	} else {
        		mTirador.setText(res.getString(R.string.acc_paso));
        		mAutoHandler.sleep(200*autointv);
        	}
    	} else {
    		mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.dadox, 0, 0, 0);
    		mTirador.getBackground().setColorFilter(0xff777777, PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(0xff777777, PorterDuff.Mode.SRC_ATOP);
    		mTirador.setText(res.getString(R.string.acc_fin));
        	mAuxiliar.invalidate();
        	Animation saltoAnimation = AnimationUtils.loadAnimation(this, R.anim.salto);
        	mParchisView.startAnimation(saltoAnimation);
    	}
    }

    private void pintaVdado(int vdado) {
		int res;
		if (vdado == 1) res = R.drawable.dado1;
		else if (vdado == 2) res = R.drawable.dado2;
		else if (vdado == 3) res = R.drawable.dado3;
		else if (vdado == 4) res = R.drawable.dado4;
		else if (vdado == 5) res = R.drawable.dado5;
		else if (vdado == 6) res = R.drawable.dado6;
		else if (vdado == 7) res = R.drawable.dado6;
		else if (vdado == 10) res = R.drawable.dado10;
		else if (vdado == 20) res = R.drawable.dado20;
		else return;
		mTirador.setCompoundDrawablesWithIntrinsicBounds(res, 0, 0, 0);
    }
    
    public void ajustaAccion() {
        if (mEP.accion == EstadoPartida.ACC_TIRADADO) {
    	    mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.mueve_dado, 0, 0, 0);
        	Drawable bdraw[] = mTirador.getCompoundDrawables();
        	AnimationDrawable dadoAnimation = (AnimationDrawable) bdraw[0];
    	    dadoAnimation.start();
        }
        if (mEP.accion == EstadoPartida.ACC_PARADADO) {
        	Drawable bdraw[] = mTirador.getCompoundDrawables();
        	AnimationDrawable dadoAnimation = (AnimationDrawable) bdraw[0];
    	    dadoAnimation.stop();
    	    pintaVdado(mEP.vdado);
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE) {
        	if (mEP.sigvdado >= 10) {
        		pintaVdado(mEP.sigvdado);
        	}
            mAuxiliar.setVisibility(View.GONE);
            mParchisView.invalidate();
        }
    }
    
    public boolean doBlinkPawn() {
    	if (blinkPawn == 0) return false;
      	mParchisView.showPawnCruz = !mParchisView.showPawnCruz;
        mParchisView.invalidate();
        if (blinkPawn > 7)
        	blinkPawn = 0;
        else
        	blinkPawn++;
        mAutoHandler.sleep(18*autointv);
        return true;
    }

    public boolean doEnCarrera() {
    	int intv = 12;
    	
    	if (!enCarrera) return false;
    	if (mCarrera == null)
    		mCarrera = new Carrera(mEP.gjg.jg[mEP.njgelegida]);
    	if (mCarrera == null || mCarrera.nula) {
    		enCarrera = false;
    		return false;
    	}
    	if (mCarrera.sigpaso >= mCarrera.npasos) {
    		if (!mCarrera.especial || mCarrera.cuentaespecial > 5) {
    		    enCarrera = false;
        	    mParchisView.modoCarrera = false;
    		    return false;
    		}
    		intv = (mCarrera.cuentaespecial % 2 == 0) ? 24 : 10;
    	}
    	mParchisView.modoCarrera = true;
    	mParchisView.setCarrera(mCarrera);
    	mParchisView.invalidate();
        mAutoHandler.sleep(intv*autointv);
        return true;
    }
    
    public void autoUpdate() {
    	if (doBlinkPawn()) return;
    	if (doEnCarrera()) return;
    	mEP.iniciaAccion(mTablero);
    	ajustaAccion();
    	mEP.terminaAccion(mTablero);
    	preparaSigAccion();
    }
    
    public void bTirador(View view) {
        Resources res = getBaseContext().getResources();

    	if (mEP.accion == EstadoPartida.ACC_PARTIDATERMINADA) {
    		CharSequence str = res.getString(R.string.toast_gameover);
    		Toast.makeText(this, str, Toast.LENGTH_SHORT).show();
    		return;
    	}
        if (mEP.tipo_jugador[mEP.turno] != EstadoPartida.PERSONA) {
        	boolean anyperson = false;
    		for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    			if (mEP.tipo_jugador[c] == EstadoPartida.PERSONA)
    				anyperson = true;
    		}
    		CharSequence str;
    		if (anyperson)
    			str = res.getString(R.string.toast_waitturn);
    		else
    			str = res.getString(R.string.toast_stopauto);
    		Toast.makeText(this, str, Toast.LENGTH_SHORT).show();
    		return;
        }
    	if (mEP.accion == EstadoPartida.ACC_PARADADO) return;

        if (doEnCarrera()) return;
    	mEP.iniciaAccion(mTablero);
    	ajustaAccion();
    	mEP.terminaAccion(mTablero);
    	preparaSigAccion();
    }

    public void bAuxiliar(View view) {
        mEP.njgelegida++;
        if (mEP.njgelegida >= mEP.gjg.njugadas) mEP.njgelegida = 0;
        mParchisView.invalidate();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.parchis4a_menu, menu);
    	menu.findItem(R.id.menu_about).setIcon(
        	getResources().getDrawable(android.R.drawable.ic_menu_info_details));
    	menu.findItem(R.id.menu_help).setIcon(
                getResources().getDrawable(android.R.drawable.ic_menu_help));
    	menu.findItem(R.id.menu_selectplayers).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_view));
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        menu.findItem(R.id.menu_fast).setIcon(getResources().getDrawable(
        	goFast ? android.R.drawable.button_onoff_indicator_on :
        	android.R.drawable.button_onoff_indicator_off));
   	return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        case R.id.menu_selectplayers:
        	showDialog(DIALOG_SELECTPLAYERS_ID);
            return true;
        case R.id.menu_fast:
        	if (goFast) {
        		goFast = false;
        		autointv = 10;
        	} else {
        		goFast = true;
        		autointv = 1;
        	}
       	    item.setIcon(getResources().getDrawable(
               	goFast ? android.R.drawable.button_onoff_indicator_on :
               	android.R.drawable.button_onoff_indicator_off));
       	    return true;
        case R.id.menu_about:
        	showDialog(DIALOG_ABOUT_ID);
            return true;
        case R.id.menu_help:
        	showDialog(DIALOG_HELP_ID);
            return true;
       default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        Dialog dialog;
        Resources res = getBaseContext().getResources();
        switch(id) {
        case DIALOG_SELECTPLAYERS_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.select_players);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(res.getString(R.string.select_players_title));
        	dialog.setTitle(res.getString(R.string.select_players_title));
        	mRG_tipo_jugador[0] = (RadioGroup) dialog.findViewById(R.id.radioGroup1);
        	mRG_tipo_jugador[1] = (RadioGroup) dialog.findViewById(R.id.radioGroup2);
        	mRG_tipo_jugador[2] = (RadioGroup) dialog.findViewById(R.id.radioGroup3);
        	mRG_tipo_jugador[3] = (RadioGroup) dialog.findViewById(R.id.radioGroup4);
        	for (int j=0; j<4; j++) {
        		if (tipo_jugador[j] == EstadoPartida.PERSONA) mRG_tipo_jugador[j].check(R.id.radio0);
        		else if (tipo_jugador[j] == EstadoPartida.MAQUINA) mRG_tipo_jugador[j].check(R.id.radio1);
        		else mRG_tipo_jugador[j].check(R.id.radio2);
        	}

            Button selectpDialogOkButton = (Button) dialog.findViewById(R.id.select_players_ok_button);
            selectpDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                	for (int j=0; j<4; j++) {
                    	int id = mRG_tipo_jugador[j].getCheckedRadioButtonId();
                		if (id == R.id.radio0) tipo_jugador[j] = EstadoPartida.PERSONA;
                		else if (id == R.id.radio1) tipo_jugador[j] = EstadoPartida.MAQUINA;
                		else tipo_jugador[j] = EstadoPartida.NOJUEGA;
                	}
                    setNewGame();
                    dismissDialog(DIALOG_SELECTPLAYERS_ID);
                }
            });

            Button selectpDialogCancelButton = (Button) dialog.findViewById(R.id.select_players_cancel_button);
            selectpDialogCancelButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_SELECTPLAYERS_ID);
                }
            });

            break;
        case DIALOG_ABOUT_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.about_layout);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(res.getString(R.string.about_title));

            Button aboutDialogOkButton = (Button) dialog.findViewById(R.id.about_ok_button);
                aboutDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                    public void onClick(View v) {
                        dismissDialog(DIALOG_ABOUT_ID);
                    }
                });
            break;
        case DIALOG_HELP_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.help_layout);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(res.getString(R.string.help_title));

            Button helpDialogOkButton = (Button) dialog.findViewById(R.id.help_ok_button);
                helpDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                    public void onClick(View v) {
                        dismissDialog(DIALOG_HELP_ID);
                    }
                });
            break;
        case DIALOG_EXIT_ID:
        	AlertDialog.Builder builder = new AlertDialog.Builder(this);
        	builder.setMessage(res.getString(R.string.dlg_exit_title))
        	       .setCancelable(false)
        	       .setPositiveButton(res.getString(R.string.dlg_yes),
        	    	   new DialogInterface.OnClickListener() {
        	           public void onClick(DialogInterface dialog, int id) {
        	               finish();
        	           }
        	       })
        	       .setNegativeButton(res.getString(R.string.dlg_no),
        	    	   new DialogInterface.OnClickListener() {
        	           public void onClick(DialogInterface dialog, int id) {
       	                   dialog.cancel();
        	           }
        	       });
        	dialog = builder.create();
        	break;
        case DIALOG_WELCOME_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.welcome_layout);
        	dialog.setCancelable(true);
        	dialog.setTitle(res.getString(R.string.welcome_title));

           Button welcomeBeginButton = (Button) dialog.findViewById(R.id.welcome_begin_button);
               welcomeBeginButton.setOnClickListener(new Button.OnClickListener() {
                   public void onClick(View v) {
                       dismissDialog(DIALOG_WELCOME_ID);
                   }
               });
               Button welcomeSelectButton = (Button) dialog.findViewById(R.id.welcome_select_button);
               welcomeSelectButton.setOnClickListener(new Button.OnClickListener() {
                   public void onClick(View v) {
                	   showDialog(DIALOG_SELECTPLAYERS_ID);
                       dismissDialog(DIALOG_WELCOME_ID);
                   }
               });
               Button welcomeHelpButton = (Button) dialog.findViewById(R.id.welcome_help_button);
               welcomeHelpButton.setOnClickListener(new Button.OnClickListener() {
                   public void onClick(View v) {
                	   showDialog(DIALOG_HELP_ID);
                       dismissDialog(DIALOG_WELCOME_ID);
                   }
               });
            break;
        default:
            dialog = null;
        }
        return dialog;
    }
    
    @Override
    public void onBackPressed() {
    	if (mEP.haComenzado() && !mEP.haTerminado())
    	    showDialog(DIALOG_EXIT_ID);
    	else
    	    finish();
    }
}